{
    SQL Browse Directory Dialog
    Copyright: Andrew Venmore (avenmore@ananzi.co.za)
    Ver: 1.0 (27 April 2004)
}

unit awvSQLBrowseDirDlg;

// ----------------------------------------------------------------------- //
                                  interface
// ----------------------------------------------------------------------- //
uses
  Windows, SysUtils, Forms, Dialogs, Contnrs,
  StdCtrls, ComCtrls, DB, ADODB, ImgList, Controls, Classes, ExtCtrls;
// ----------------------------------------------------------------------- //

type
  TawvSQLBrowseDirDlg = class(TComponent)
  private
    FConnectionString: string;
    FServer: string;
    FLoginName: string;
    FLoginPass: string;
    FFileName: string;
    FCaption: string;
    FShowFiles: boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: boolean;
  published
    property ConnectionString: string read FConnectionString write FConnectionString;
    property Server: string read FServer write FServer;
    property LoginName: string read FLoginName write FLoginName;
    property LoginPass: string read FLoginPass write FLoginPass;
    property FileName: string read FFileName write FFileName;
    property Caption: string read FCaption write FCaption;
    property ShowFiles: boolean read FShowFiles write FShowFiles default true;
  end;

// ----------------------------------------------------------------------- //

  TTreeNodeDirData = class(TObject)
  private
    DisplayValue: string;
    FullName: string;
    IsFolder: boolean;
    Built: boolean;
  public  
    constructor Create(const ADisplayValue, AFullName: string; const AIsFolder, ABuilt: boolean);
    destructor Destroy; override;
  end;

// ----------------------------------------------------------------------- //

  TfrmSQLBrowseDirDlg = class(TForm)
    pnlButtons: TPanel;
    btnCancel: TButton;
    btnOK: TButton;
    pnlMain: TPanel;
    lblFileNameCapt: TLabel;
    lblSelectedCapt: TLabel;
    imlTree: TImageList;
    adoSQLDirBrowse: TADOConnection;
    spxp_fixeddrives: TADOStoredProc;
    tlFolders: TTreeView;
    edtFileName: TEdit;
    lblSelectedFile: TLabel;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure tlFoldersExpanding(Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
    procedure tlFoldersChange(Sender: TObject; Node: TTreeNode);
    procedure FormDestroy(Sender: TObject);
    procedure edtFileNameChange(Sender: TObject);
  private
    FawvSQLBrowseDirDlg: TawvSQLBrowseDirDlg;
    SettingEditValue: boolean;
    DataList: TObjectlist;
    function ValidConnection: boolean;
    procedure BuildTree;
    procedure BuildSubItems(const ANode: TTreeNode; const Depth: integer);
    procedure FindSelectedNode;
    procedure SetawvSQLBrowseDirDlg(const Value: TawvSQLBrowseDirDlg);
    property awvSQLBrowseDirDlg: TawvSQLBrowseDirDlg read FawvSQLBrowseDirDlg write SetawvSQLBrowseDirDlg;
  end;

// ----------------------------------------------------------------------- //
                               implementation
{$R *.dfm}
{$R awvSQLBrowseDirDlg.res}
// ----------------------------------------------------------------------- //
uses
  awvSQLBrowseDirUtils;
// ----------------------------------------------------------------------- //

{ TawvSQLBrowseDirDlg }

constructor TawvSQLBrowseDirDlg.Create(AOwner: TComponent);
begin
  inherited;
  FShowFiles := true;
end;

destructor TawvSQLBrowseDirDlg.Destroy;
begin
  inherited;
end;

function TawvSQLBrowseDirDlg.Execute: boolean;
var
  frmSQLBrowseDirDlg: TfrmSQLBrowseDirDlg;
begin
  PushCursor;
  frmSQLBrowseDirDlg := TfrmSQLBrowseDirDlg.Create(Application);
  frmSQLBrowseDirDlg.awvSQLBrowseDirDlg := Self;
  PopCursor;
  Result := (frmSQLBrowseDirDlg.ValidConnection) and (frmSQLBrowseDirDlg.ShowModal = mrOK);
  frmSQLBrowseDirDlg.Free;
end;

// ----------------------------------------------------------------------- //

{ TTreeNodeDirData }

constructor TTreeNodeDirData.Create(const ADisplayValue, AFullName: string;
  const AIsFolder, ABuilt: boolean);
begin
  DisplayValue := ADisplayValue;
  FullName := AFullName;
  IsFolder := AIsFolder;
  Built := ABuilt;
end;

destructor TTreeNodeDirData.Destroy;
begin
  inherited;
end;

// ----------------------------------------------------------------------- //

{ TfrmSQLBrowseDirDlg }

procedure TfrmSQLBrowseDirDlg.FormCreate(Sender: TObject);
begin
  SettingEditValue := false;
  DataList := TObjectlist.Create;
end;

procedure TfrmSQLBrowseDirDlg.FormDestroy(Sender: TObject);
begin
  DataList.Free;
end;

procedure TfrmSQLBrowseDirDlg.SetawvSQLBrowseDirDlg(const Value: TawvSQLBrowseDirDlg);
begin
  FawvSQLBrowseDirDlg := Value;
  if (not FawvSQLBrowseDirDlg.ShowFiles) then begin
    lblFileNameCapt.Visible := false;
    edtFileName.Visible := false;
    tlFolders.Height := tlFolders.Height + 34;
    lblSelectedCapt.Caption := 'Selected Folder:';
  end;
end;

procedure TfrmSQLBrowseDirDlg.FormShow(Sender: TObject);
begin
  PushCursor;
  if (awvSQLBrowseDirDlg.Caption > '') then
    Caption := awvSQLBrowseDirDlg.Caption;
  lblSelectedFile.Caption := '';
  BuildTree;
  FindSelectedNode;
  PopCursor;
end;

function TfrmSQLBrowseDirDlg.ValidConnection: boolean;
begin
  PushCursor;
  try
    if (awvSQLBrowseDirDlg.ConnectionString > '') then
      adoSQLDirBrowse.ConnectionString := awvSQLBrowseDirDlg.ConnectionString
    else
      adoSQLDirBrowse.ConnectionString := BuildConnectionString(awvSQLBrowseDirDlg.Server, awvSQLBrowseDirDlg.LoginName, awvSQLBrowseDirDlg.LoginPass, '');
    adoSQLDirBrowse.Connected := true;
    Result := true;
  except
    on E:Exception do begin
      MessageDlg(E.Message, mtError, [mbOK], 0);
      Result := false;
    end;
  end;
  PopCursor;
end;

procedure TfrmSQLBrowseDirDlg.BuildTree;
var
  NewNode: TTreeNode;
  NewNodeData: TTreeNodeDirData;
  i: integer;
begin
  PushCursor;
  tlFolders.Items.Clear;

  spxp_fixeddrives.Open;
  while (not spxp_fixeddrives.Eof) do begin
    NewNodeData := TTreeNodeDirData.Create(Format('%s:\ (%d MB free)', [spxp_fixeddrives.FieldByName('drive').AsString, spxp_fixeddrives.FieldByName('MB free').AsInteger]), Format('%s:\', [spxp_fixeddrives.FieldByName('drive').AsString]), true, false);
    NewNode := tlFolders.Items.AddChildObject(nil, NewNodeData.DisplayValue, NewNodeData);
    DataList.Add(NewNodeData); // To make sure object is freed
    NewNode.ImageIndex := 0;
    NewNode.SelectedIndex := 1;
    spxp_fixeddrives.Next;
  end;
  spxp_fixeddrives.Close;

  for i := 0 to tlFolders.Items.Count - 1 do
    BuildSubItems(tlFolders.Items[i], 2);

  PopCursor;
end;

procedure TfrmSQLBrowseDirDlg.FindSelectedNode;

    function FindChildWithPath(const ANode: TTreeNode; const APath: string): TTreeNode;
    var
      i: integer;
    begin
      Result := nil;
      if (ANode = nil) then begin
        for i := 0 to tlFolders.Items.Count - 1 do
          if (SameText(NoBackSlash(TTreeNodeDirData(tlFolders.Items[i].Data).FullName), APath)) then begin
            Result := tlFolders.Items[i];
            Break;
          end;
      end
      else begin
        for i := 0 to ANode.Count - 1 do
          if (SameText(NoBackSlash(TTreeNodeDirData(ANode.Item[i].Data).DisplayValue), APath)) then begin
            Result := ANode.Item[i];
            Break;
          end;
      end;
    end;

var
  NodeFound: boolean;
  SL: TStringlist;
  i: integer;
  CurNode: TTreeNode;
begin
  if (tlFolders.Items.Count > 0) then begin
    NodeFound := false;

    if (awvSQLBrowseDirDlg.FileName > '') then begin
      SL := TStringlist.Create;
      SplitString(awvSQLBrowseDirDlg.FileName, '\', SL);
      CurNode := nil;
      for  i := 0 to SL.Count - 1 do begin
        CurNode := FindChildWithPath(CurNode, SL[i]);
        if (Assigned(CurNode)) then begin
          CurNode.Focused := true;
          CurNode.Selected := true;
          CurNode.MakeVisible;
          CurNode.Expand(false);
          NodeFound := true;
        end
        else
          Break;
      end;
      SL.Free;
    end;

    if (not NodeFound) then begin
      tlFolders.Items[0].Focused := true;
      tlFolders.Items[0].Selected := true;
      tlFolders.Items[0].Expand(false);
      tlFolders.Items[0].MakeVisible;
    end;
  end;

  // Assume file specified if has ext - not perfect
  if (FawvSQLBrowseDirDlg.ShowFiles) and (awvSQLBrowseDirDlg.FileName > '') and (ExtractFileExt(awvSQLBrowseDirDlg.FileName) > '') then
    edtFileName.Text := ExtractFileName(awvSQLBrowseDirDlg.FileName);
end;

procedure TfrmSQLBrowseDirDlg.BuildSubItems(const ANode: TTreeNode; const Depth: integer);
var
  NewNode: TTreeNode;
  NewNodeData: TTreeNodeDirData;
  spxp_dirtree: TADOStoredProc;
  i: integer;
begin
  if (TTreeNodeDirData(ANode.Data).IsFolder) then begin
    if (not TTreeNodeDirData(ANode.Data).Built) then begin
      PushCursor;
      spxp_dirtree := TADOStoredProc.Create(nil);
      spxp_dirtree.Connection := adoSQLDirBrowse;
      spxp_dirtree.ProcedureName := 'master..xp_dirtree';
      spxp_dirtree.Parameters.AddParameter.Value := TTreeNodeDirData(ANode.Data).FullName;
      spxp_dirtree.Parameters.AddParameter.Value := 1;
      if (awvSQLBrowseDirDlg.ShowFiles) then
        spxp_dirtree.Parameters.AddParameter.Value := 1;
      spxp_dirtree.Open;
      // Do folders first
      while (not spxp_dirtree.Eof) do begin
        if (not awvSQLBrowseDirDlg.ShowFiles) or (spxp_dirtree.FieldByName('file').AsInteger = 0) then begin
          NewNodeData := TTreeNodeDirData.Create(spxp_dirtree.FieldByName('subdirectory').AsString, BackSlash(TTreeNodeDirData(ANode.Data).FullName) + spxp_dirtree.FieldByName('subdirectory').AsString, true, false);
          NewNode := tlFolders.Items.AddChildObject(ANode, NewNodeData.DisplayValue, NewNodeData);
          DataList.Add(NewNodeData); // To make sure object is freed
          NewNode.ImageIndex := 0;
          NewNode.SelectedIndex := 1;
          if (Depth = 2) then
            BuildSubItems(NewNode, 1); // Recurse
        end;
        spxp_dirtree.Next;
      end;
      if (awvSQLBrowseDirDlg.ShowFiles) then begin
        // Then files
        spxp_dirtree.First;
        while (not spxp_dirtree.Eof) do begin
          if (spxp_dirtree.FieldByName('file').AsInteger > 0) then begin
            NewNodeData := TTreeNodeDirData.Create(spxp_dirtree.FieldByName('subdirectory').AsString, BackSlash(TTreeNodeDirData(ANode.Data).FullName) + spxp_dirtree.FieldByName('subdirectory').AsString, false, false);
            NewNode := tlFolders.Items.AddChildObject(ANode, NewNodeData.DisplayValue, NewNodeData);
            DataList.Add(NewNodeData); // To make sure object is freed
            NewNode.ImageIndex := 2;
            NewNode.SelectedIndex := 2;
          end;
          spxp_dirtree.Next;
        end;
      end;
      spxp_dirtree.Close;
      spxp_dirtree.Free;
      TTreeNodeDirData(ANode.Data).Built := true;
      PopCursor;
    end
    else begin
      if (Depth = 2) then
        for i := 0 to ANode.Count - 1 do
          BuildSubItems(ANode.Item[i], 1); // Recurse
    end;
  end;
end;

procedure TfrmSQLBrowseDirDlg.edtFileNameChange(Sender: TObject);
begin
  if (not SettingEditValue) then begin
    if (Assigned(tlFolders.Selected)) then begin
      if (not TTreeNodeDirData(tlFolders.Selected.Data).IsFolder) then
        lblSelectedFile.Caption := BackSlash(TTreeNodeDirData(tlFolders.Selected.Parent.Data).FullName) + edtFileName.Text
      else
      if (edtFileName.Text > '') then
        lblSelectedFile.Caption := BackSlash(TTreeNodeDirData(tlFolders.Selected.Data).FullName) + edtFileName.Text
      else
        lblSelectedFile.Caption := TTreeNodeDirData(tlFolders.Selected.Data).FullName;
    end
    else
      lblSelectedFile.Caption := edtFileName.Text;
  end;
end;

procedure TfrmSQLBrowseDirDlg.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if (ModalResult = mrOK) Then begin
    if (Assigned(tlFolders.Selected)) then begin
      if (not TTreeNodeDirData(tlFolders.Selected.Data).IsFolder) then
        awvSQLBrowseDirDlg.FileName := BackSlash(TTreeNodeDirData(tlFolders.Selected.Parent.Data).FullName) + edtFileName.Text
      else
      if (edtFileName.Text > '') then
        awvSQLBrowseDirDlg.FileName := BackSlash(TTreeNodeDirData(tlFolders.Selected.Data).FullName) + edtFileName.Text
      else
        awvSQLBrowseDirDlg.FileName := TTreeNodeDirData(tlFolders.Selected.Data).FullName;
    end
    else
      awvSQLBrowseDirDlg.FileName := edtFileName.Text;
  end;
end;

procedure TfrmSQLBrowseDirDlg.tlFoldersExpanding(Sender: TObject;
  Node: TTreeNode; var AllowExpansion: Boolean);
begin
  BuildSubItems(Node, 2);
end;

procedure TfrmSQLBrowseDirDlg.tlFoldersChange(Sender: TObject;
  Node: TTreeNode);
begin
  if (Assigned(tlFolders.Selected)) then begin
    if (not TTreeNodeDirData(tlFolders.Selected.Data).IsFolder) then begin
      SettingEditValue := true;
      edtFileName.Text := TTreeNodeDirData(tlFolders.Selected.Data).DisplayValue;
      lblSelectedFile.Caption := TTreeNodeDirData(tlFolders.Selected.Data).FullName;
      SettingEditValue := false;
    end
    else
    if (edtFileName.Text > '') then
      lblSelectedFile.Caption := BackSlash(TTreeNodeDirData(tlFolders.Selected.Data).FullName) + edtFileName.Text
    else
      lblSelectedFile.Caption := TTreeNodeDirData(tlFolders.Selected.Data).FullName;
  end
  else
    lblSelectedFile.Caption := edtFileName.Text;
end;

end.

